"""
--------------------------- damma ------------------------------
Downscaling com aprendizado de maquinas para modelos atmosfericos
Downscaling Atmospheric Model with machine learning

modulo para downscaling de campos de temperatura provindos do wrf utilizando machine learning

"""

from .preprocessing import *
#from .train_preprocessing import *
#from .Train import *
from .posprocessing import *
from .estatisticas import *
from .downscaling import *
#from .DammaRun import *